#!/usr/bin/env python
# coding: utf-8

# In[ ]:


import tensorflow as tf
print(tf.test.is_gpu_available())


# In[ ]:


import keras
from keras.backend.tensorflow_backend import set_session
tf_config = tf.ConfigProto()
tf_config.gpu_options.per_process_gpu_memory_fraction = 0.5 # 50% GPU memory usage at most
tf_config.gpu_options.allow_growth = True
set_session(tf.Session(config=tf_config))


# In[ ]:


get_ipython().run_line_magic('matplotlib', 'inline')
from keras.layers import Dense
from keras import Sequential
import numpy as np
import matplotlib.pyplot as plt
import keras.datasets.mnist as mnist

#Define hyperparameter
EPOCHS = 20
LEARNING_RATE=0.01
BATCH_SIZE = 200


# In[ ]:


# Read MNIST dataset
# mnist.load_data() method returns a tuple. Take a look here if you are confused about it
# https://www.tensorflow.org/api_docs/python/tf/keras/datasets/mnist/load_data
(x_train, y_train), (x_test, y_test) = mnist.load_data()
print(x_train.shape)
print(y_train.shape)
print(x_test.shape)
print(y_test.shape)
plt.imshow(x_train[0], "gray")


# In[ ]:


# Data preprocess
# Change data type from uint8 to float32
# Because we is going to do normalization
x_train = x_train.astype("float32")
x_test = x_test.astype("float32")
# Normalization
x_train = x_train/255
x_test = x_test/255
# Flatten the array
x_train = x_train.reshape(x_train.shape[0], 28*28)
x_test = x_test.reshape(x_test.shape[0], 28*28)
# One-hot encode the labels
y_train = keras.utils.to_categorical(y_train, 10)
y_test = keras.utils.to_categorical(y_test, 10)



# In[ ]:


# Build the model
# Input shape needs to be specified in the first layer
# https://www.tensorflow.org/api_docs/python/tf/keras/layers/Dense
my_model = Sequential()
my_model.add(Dense(input_shape = (784, ), units = 128, activation = "sigmoid"))
my_model.add(Dense(units = 10, activation = "softmax"))
my_model.summary()


# In[ ]:


# Configure the model for training
my_model.compile(optimizer=keras.optimizers.SGD(lr=LEARNING_RATE), 
                 loss="categorical_crossentropy", metrics=["acc"])

# Feed the input data to this model and start training 
Train_history = my_model.fit(x=x_train, y=y_train, 
                       batch_size=BATCH_SIZE,
                       epochs=EPOCHS, 
                       verbose=1,
                       validation_split=0.2)


# In[ ]:


# Plot the training process
# Plot Accuracy curve
xaxis = np.linspace(1, EPOCHS, EPOCHS)
plt.figure()
plt.plot(Train_history.history["acc"])
plt.title("Training Process - Acc")
plt.xlabel("Epoch")
plt.ylabel("Accuracy")
plt.xticks(xaxis)
plt.ylim(0, 1)
plt.show()
# Plot Loss curve
plt.figure()
plt.plot(Train_history.history["loss"])
plt.title("Training Process - Loss")
plt.xlabel("Epoch")
plt.xticks(xaxis)
plt.ylabel("Loss")
plt.show()


# In[ ]:


# Evaluation
Test_result = my_model.evaluate(x = x_test, y = y_test, batch_size = x_test.shape[0])
print("Loss on testing set: %f, Accuracy on testing set: %f" % (Test_result[0], Test_result[1]))

# Save the model
my_model.save("MyFirstModel.h5")


# In[ ]:


get_ipython().run_line_magic('matplotlib', 'inline')
import keras.datasets.mnist as mnist
import matplotlib.pyplot as plt
import numpy as np
retreived_model = keras.models.load_model("./MyFirstModel.h5")
(x_train, y_train), (x_test, y_test) = mnist.load_data()
plt.figure()
plt.imshow(x_train[5], "gray")
test_image = x_train[5]
test_image = test_image.reshape(1,784)
prediction = retreived_model.predict(test_image)
print("Output: ", np.argmax(prediction))


# In[ ]:




